// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

type TGdProviderReport struct {
	Id              int64
	Date            string `json:"date"`
	ProviderName    string `json:"provider_name"`
	ProviderApiName string `json:"provider_api_name"`
	Total           int64  `json:"total"`
	Success         int64  `json:"success"`
	SuccessRate     string
	Failed          int64  `json:"failed"`
	FailedRate      string `json:"failed_rate"`
	Query           int64  `json:"query"`
	QueryRate       string `json:"query_no_record_rate"`
	AvgElapsed      string `json:"avg_elapsed"`
	BigElapsedList  string `json:"big_elapsed_list"`
	Charge          int64  `json:"charge"`
	ChargeRate      string `json:"charge_rate"`
	ProviderApiId   int64
}

func getTodayProviderReport(req *apis.BiGetProviderReportReq) ([]apis.BiProviderReportInfo, error) {
	mreq := apis.LogQueryProviderCountExportReq{}

	mreq.ProviderApiIdList = req.ProviderApiId
	mreq.TabName = "t_gd_thirdpart_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = getTodayTimestamp()
	if req.TodayEndTimestamp > 0 {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryProviderCountExportReply{}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := make([]apis.BiProviderReportInfo, len(mreply.LogQueryProviderCount))
	for i, v := range mreply.LogQueryProviderCount {
		item := apis.BiProviderReportInfo{}
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.AvgElapsed = strconv.FormatFloat(v.AvgElapsed, 'f', 2, 64)
		item.Fail = v.Failed
		item.FailRate = v.FailedRate
		item.ProviderApiName = v.ProviderApiName
		item.ProviderName = v.ProviderName
		item.QueryNoRecord = v.Query
		item.ProviderApiId = v.ProviderApiId
		item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Total), 'f', 2, 64) + "%"
		ret[i] = item
	}
	return ret, nil
}

func getTodayProviderReportFromHour(req *apis.BiGetProviderReportReq, start, end int64) ([]apis.BiProviderReportInfo, error) {
	if req.TodayEndTimestamp > 0 && req.TodayEndTimestamp < end {
		end = req.TodayEndTimestamp
	}
	ret := []apis.BiProviderReportInfo{}

	o := orm.NewOrm()
	req.ProviderApiIdNew = req.ProviderApiId
	sql := fmt.Sprintf("select provider_name, provider_api_name, provider_api_id, sum(charge) as charge, sum(total) as total,"+
		"sum(sum_elapsed) as sum_elapsed, sum(query) as query_no_record, sum(failed) as fail from t_gd_provider_report_hour "+
		"where hour >= %d and hour < %d", start, end)
	if len(req.ProviderApiIdNew) > 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by provider_api_id", sql)
	if _, err := o.Raw(sql).QueryRows(&ret); err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	newList := []apis.BiProviderReportInfo{}
	for i, v := range ret {
		ret[i].Date = time.Now().Format("2006-01-02")
		ret[i].ChargeRate = strconv.FormatFloat(float64(100*v.Charge)/float64(v.Total), 'f', 2, 64) + "%"
		ret[i].AvgElapsed = strconv.FormatFloat(v.SumElapsed/float64(v.Total), 'f', 2, 64)
		ret[i].FailRate = strconv.FormatFloat(float64(100*v.Fail)/float64(v.Total), 'f', 2, 64)
		ret[i].FailRate = strconv.FormatFloat(float64(100*v.Fail)/float64(v.Total), 'f', 2, 64) + "%"
		ret[i].QueryNoRecordRate = strconv.FormatFloat(float64(100*v.QueryNoRecord)/float64(v.Total), 'f', 2, 64) + "%"
		newList = append(newList, ret[i])
	}
	return newList, nil
}

func parseProviderToday(req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply) error {
	infos, err := getTodayProviderReport(req)
	newList := []apis.BiProviderReportInfo{}
	if err != nil {
		return err
	}
	for _, v := range infos {
		newList = append(newList, v)
	}
	reply.Infos = newList
	return nil
}

func parseProviderTodayByApi(req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply) error {
	infos, err := getTodayProviderReport(req)
	newList := []apis.BiProviderReportInfo{}
	if err != nil {
		return err
	}
	for _, v := range infos {
		newList = append(newList, v)
	}
	infos = newList
	return nil
}

func parseProviderOldDay(o orm.Ormer, req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply, dates string) error {
	tabs := []TGdProviderReport{}
	sql := fmt.Sprintf("select * from t_gd_provider_report where date in(%s) ", dates)
	if len(req.ProviderApiIdNew) != 0 {
		ids := ""
		for _, v := range req.ProviderApiIdNew {
			if ids == "" {
				ids = fmt.Sprintf("%d", v)
			} else {
				ids = fmt.Sprintf("%s,%d", ids, v)
			}
		}
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, ids)
	}
	sql = fmt.Sprintf("%s order by date, provider_api_id", sql)
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	infos := make([]apis.BiProviderReportInfo, len(tabs))
	for i, v := range tabs {
		item := apis.BiProviderReportInfo{}
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.AvgElapsed = v.AvgElapsed
		item.Fail = v.Failed
		item.FailRate = v.FailedRate
		item.ProviderName = v.ProviderName
		item.ProviderApiName = v.ProviderApiName
		item.QueryNoRecord = v.Query
		item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Total), 'f', 2, 64) + "%"
		infos[i] = item
	}
	reply.Infos = append(reply.Infos, infos...)

	return nil
}

func parseProviderOldDayByApi(o orm.Ormer, req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply, dates string) error {
	tabs := []TGdProviderReport{}
	sql := fmt.Sprintf("select * from t_gd_provider_report where date in(%s) ", dates)
	if len(req.ProviderApiIdNew) != 0 {
		ids := ""
		for _, v := range req.ProviderApiIdNew {
			if ids == "" {
				ids = fmt.Sprintf("%d", v)
			} else {
				ids = fmt.Sprintf("%s,%d", ids, v)
			}
		}
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, ids)
	}
	sql = fmt.Sprintf("%s order by date, provider_api_id", sql)
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	infos := make([]apis.BiProviderReportInfo, len(tabs))
	for i, v := range tabs {
		item := apis.BiProviderReportInfo{}
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.AvgElapsed = v.AvgElapsed
		item.Fail = v.Failed
		item.FailRate = v.FailedRate
		item.ProviderName = v.ProviderName
		item.ProviderApiName = v.ProviderApiName
		item.QueryNoRecord = v.Query
		item.ProviderApiId = v.ProviderApiId
		item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Total), 'f', 2, 64) + "%"
		infos[i] = item
	}
	reply.Infos = append(reply.Infos, infos...)

	return nil
}

func BiGetProviderReportByApi(req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply) (err error) {
	if len(req.Date) == 0 {
		return errors.ArgsError
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	offset := (req.PageNumber - 1) * pageSize
	limit := pageSize

	// parse date to sql string
	dates, containToday := parseDate(req.Date)

	// get total count
	o := orm.NewOrm()
	req.ProviderApiIdNew = req.ProviderApiId
	if containToday {
		err = parseProviderTodayByApi(req, reply)
		if err != nil {
			return err
		}
	}

	err = parseProviderOldDayByApi(o, req, reply, dates)
	if err != nil {
		return err
	}
	reply.PageSize = pageSize

	infos := make([]apis.BiProviderReportInfo, limit)
	m := map[int64]*apis.BiProviderReportInfo{}
	oldSumElapsed := float64(0)
	for i, v := range reply.Infos {
		key := v.ProviderApiId
		if _, ok := m[key]; ok == false {
			m[key] = &reply.Infos[i]
			continue
		}
		//处理平均时间
		newAvgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		newSumElapsed := newAvgElapsed * float64(v.Total)
		if oldSumElapsed == 0 {
			oldAvgElapsed, _ := strconv.ParseFloat(m[key].AvgElapsed, 64)
			oldSumElapsed = oldAvgElapsed * float64(m[key].Total)
		}
		m[key].AvgElapsed = fmt.Sprintf("%f", (newSumElapsed+oldSumElapsed)/float64(m[key].Total+v.Total))
		oldSumElapsed = newSumElapsed + oldSumElapsed

		// 部分指标累加
		m[key].Total += v.Total
		m[key].Charge += v.Charge
		m[key].QueryNoRecord += v.QueryNoRecord
		m[key].Fail += v.Fail
	}

	i := 0
	count := 0
	reply.Total = len(m)
	for _, v := range m {
		if i < offset {
			i++
			continue
		}
		if count >= limit {
			break
		}
		item := apis.BiProviderReportInfo{}
		item.Total = v.Total
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = strconv.FormatFloat(float64(100*item.Charge)/float64(item.Total), 'f', 2, 64) + "%"
		avgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		item.AvgElapsed = strconv.FormatFloat(avgElapsed, 'f', 2, 64)
		item.Fail = v.Fail
		item.FailRate = strconv.FormatFloat(float64(100*item.Fail)/float64(item.Total), 'f', 2, 64) + "%"
		item.ProviderApiName = v.ProviderApiName
		item.QueryNoRecord = v.QueryNoRecord
		item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Total), 'f', 2, 64) + "%"
		infos[count] = item
		i++
		count++
	}
	reply.Infos = infos[:count]
	return nil

}

func sortProviderReport(array []apis.BiProviderReportInfo) []apis.BiProviderReportInfo {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Total < array[j].Total {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	return array

}

func sumProviderReport(infos []apis.BiProviderReportInfo) apis.BiProviderReportInfo {
	ret := apis.BiProviderReportInfo{}
	sumElapsed := float64(0)
	if len(infos) == 0 {
		return ret
	}
	for _, v := range infos {
		ret.Total += v.Total
		ret.Charge += v.Charge
		ret.Fail += v.Fail
		ret.QueryNoRecord += v.QueryNoRecord
		avgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		sumElapsed += (avgElapsed * float64(v.Total))
	}
	ret.AvgElapsed = strconv.FormatFloat(sumElapsed/(float64(ret.Total)), 'f', 2, 64)
	ret.ChargeRate = strconv.FormatFloat(float64(100*ret.Charge)/float64(ret.Total), 'f', 2, 64) + "%"
	fmt.Printf("charge rate:%v,%v,%v\n", ret.Charge, ret.Total, ret.ChargeRate)
	ret.FailRate = strconv.FormatFloat(float64(100*ret.Fail)/float64(ret.Total), 'f', 2, 64) + "%"
	ret.QueryNoRecordRate = strconv.FormatFloat(float64(100*ret.QueryNoRecord)/float64(ret.Total), 'f', 2, 64) + "%"
	return ret
}

func BiGetProviderReport(ctx context.Context, req *apis.BiGetProviderReportReq, reply *apis.BiGetProviderReportReply) (err error) {
	if len(req.Date) == 0 {
		return errors.ArgsError
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	if req.PageSize == 0 {
		req.PageSize = pageSize
	}
	offset := (req.PageNumber - 1) * req.PageSize
	limit := req.PageSize

	// parse date to sql string
	dates, containToday := parseDate(req.Date)

	// get total count
	o := orm.NewOrm()
	req.ProviderApiIdNew = req.ProviderApiId
	if containToday {
		err = parseProviderToday(req, reply)
		if err != nil {
			return err
		}
	}

	if err := parseProviderOldDay(o, req, reply, dates); err != nil {
		return err
	}
	newInfos := []apis.BiProviderReportInfo{}
	for _, v := range reply.Infos {
		if (v.ProviderName == "A02-MS" || strings.Contains(v.ProviderName, "觅实")) &&
			strings.Contains(v.ProviderApiName, "MSN") == false && strings.Contains(v.ProviderApiName, "MSM") == false {
			continue
		}
		newInfos = append(newInfos, v)
	}
	reply.Infos = newInfos
	reply.Infos = sortProviderReport(reply.Infos)
	reply.Sum = sumProviderReport(reply.Infos)
	reply.Total = len(reply.Infos)
	return nil
	if req.IsAll {
		return nil
	}
	switch {
	case reply.Total-offset <= 0:
		reply.Infos = []apis.BiProviderReportInfo{}
	case reply.Total-offset <= limit:
		reply.Infos = reply.Infos[offset:]
	case reply.Total-offset > limit:
		reply.Infos = reply.Infos[offset : offset+limit]
	}
	return nil
}
