// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"strings"
	"time"
)

func getProviderServiceStatusFromReport(req *apis.BiGetProviderServiceStatusReq, dates string, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if dates == "" {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(avg_elapsed * total) as sum_elapsed, sum(total) as count, sum(failed) as fail_count, date from t_gd_provider_report where date in(%s)", dates)
	if len(req.ProviderApiIdNew) > 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by date", sql)
	_, err := o.Raw(sql).QueryRows(&ret)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func getProviderServiceStatusFromHourReport(req *apis.BiGetProviderServiceStatusReq, hours string, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if hours == "" {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(sum_elapsed) as sum_elapsed, sum(total) as count, sum(failed) as fail_count, FROM_UNIXTIME(hour) as date from t_gd_provider_report_hour where hour in(%s)", hours)
	if len(req.ProviderApiIdNew) > 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by hour", sql)
	_, err := o.Raw(sql).QueryRows(&ret)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func getProviderServiceStatusFromDayLog(req *apis.BiGetProviderServiceStatusReq, start, end int64, o orm.Ormer, isHour bool) (*apis.ServiceStatus, error) {
	ret := &apis.ServiceStatus{}

	mreq := apis.LogQueryProviderCountExportReq{}

	mreq.ProviderApiIdList = req.ProviderApiId
	mreq.TabName = "t_gd_thirdpart_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = start, end
	mreq.NoCharge = true
	mreply := apis.LogQueryProviderCountExportReply{}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}

	for _, v := range mreply.LogQueryProviderCount {
		if (strings.Contains(v.ProviderName, "觅实") || strings.Contains(v.ProviderName, "A02")) && strings.Contains(v.ProviderApiName, "觅实N0") == false && strings.Contains(v.ProviderApiName, "MSM00001") == false && strings.Contains(v.ProviderApiName, "MSN0") == false {
			continue
		}
		ret.SumElapsed += v.SumElapsed
		ret.Count += float64(v.Total)
		ret.FailCount += int(v.Failed)
	}
	ret.Date = time.Unix(start, 0).Format("2006-01-02")
	if isHour {
		ret.Date = time.Unix(start, 0).Format("2006-01-02 15:04:05")
	}
	return ret, nil
}

func getProviderServiceStatusFromDayLogByHour(req *apis.BiGetProviderServiceStatusReq, start, end int64, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if start == 0 || end == 0 {
		return ret, nil
	}
	for i := start; i < end; i += 3600 {
		e := start + 3600
		if i+3600 > e {
			e = end
		}
		info, err := getProviderServiceStatusFromDayLog(req, i, e, o, true)
		if err != nil {
			return nil, err
		}
		ret = append(ret, *info)
	}
	return ret, nil
}

func combineProviderServiceStatusDay(old []apis.ServiceStatus, today *apis.ServiceStatus, dateArray []string) []apis.ServiceStatus {
	m := map[string]*apis.ServiceStatus{}
	for i, v := range old {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &old[i]
		}
	}
	if today.Date != "" {
		m[today.Date] = today
	}
	ret := make([]apis.ServiceStatus, len(dateArray))
	for i, v := range dateArray {
		if _, ok := m[v]; ok == false {
			item := apis.ServiceStatus{}
			item.TimestampMinute = DateToTimestamp(v)
			ret[i] = item
			continue
		}
		item := apis.ServiceStatus{}
		item.TimestampMinute = DateToTimestamp(v)
		item.FailCount = m[v].FailCount
		if m[v].Count > 0 {
			item.AvgElapsed = m[v].SumElapsed / m[v].Count
		} else {
			item.AvgElapsed = 0
		}
		ret[i] = item
	}
	return ret
}

func combineProviderServiceStatusHour(old []apis.ServiceStatus, newInfo []apis.ServiceStatus, hourArray []string) []apis.ServiceStatus {
	m := map[string]*apis.ServiceStatus{}
	for i, v := range old {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &old[i]
		}
	}
	for i, v := range newInfo {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &newInfo[i]
		}
	}

	ret := make([]apis.ServiceStatus, len(hourArray))
	for i, v := range hourArray {
		if _, ok := m[v]; ok == false {
			item := apis.ServiceStatus{}
			item.TimestampMinute = HourToTimestamp(v)
			ret[i] = item
			continue
		}
		item := apis.ServiceStatus{}
		item.TimestampMinute = HourToTimestamp(v)
		item.FailCount = m[v].FailCount
		if m[v].Count > 0 {
			item.AvgElapsed = m[v].SumElapsed / m[v].Count
		} else {
			item.AvgElapsed = 0
		}
		ret[i] = item
	}
	return ret
}

func getProviderServiceStatusByDay(req *apis.BiGetProviderServiceStatusReq, reply *apis.BiGetProviderServiceStatusReply) error {
	dateArray := getDatesFromTimestamp(req.StartTimestamp, req.EndTimestamp)
	now := time.Now()
	dates := ""
	todayZero := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
	o := orm.NewOrm()

	req.ProviderApiIdNew = req.ProviderApiId
	containToday := false
	for _, v := range dateArray {
		if v == now.Format("2006-01-02") {
			containToday = true
			continue
		}
		if dates == "" {
			dates = fmt.Sprintf("'%s'", v)
			continue
		}
		dates = fmt.Sprintf("%s, '%s'", dates, v)
	}

	oldInfo, err := getProviderServiceStatusFromReport(req, dates, o)
	if err != nil {
		return err
	}
	todayInfo := &apis.ServiceStatus{}
	if containToday {
		todayInfo, err = getProviderServiceStatusFromDayLog(req, todayZero.Unix(), req.EndTimestamp, o, false)
		if err != nil {
			return err
		}
	}
	reply.Infos = combineProviderServiceStatusDay(oldInfo, todayInfo, dateArray)
	return nil
}

func getProviderServiceStatusByHour(req *apis.BiGetProviderServiceStatusReq, reply *apis.BiGetProviderServiceStatusReply) error {
	o := orm.NewOrm()
	req.ProviderApiIdNew = req.ProviderApiId
	tabs := []string{
		"t_gd_provider_report_hour",
	}
	hourStart, hourEnd, otherStart, otherEnd, err := parseHourTimestamp(o, req.StartTimestamp, req.EndTimestamp, tabs)
	if err != nil {
		return err
	}
	_, hours := getHoursByTimestamp(hourStart, hourEnd)
	hourArray, _ := getHoursByTimestamp(req.StartTimestamp, req.EndTimestamp)
	oldInfo, err := getProviderServiceStatusFromHourReport(req, hours, o)
	if err != nil {
		return err
	}
	newInfo, err := getProviderServiceStatusFromDayLogByHour(req, otherStart, otherEnd, o)
	if err != nil {
		return err
	}

	reply.Infos = combineProviderServiceStatusHour(oldInfo, newInfo, hourArray)
	return nil
}

func BiGetProviderServiceStatus(ctx context.Context, req *apis.BiGetProviderServiceStatusReq, reply *apis.BiGetProviderServiceStatusReply) error {
	if req.TodayEndTimestamp > 0 && req.EndTimestamp > req.TodayEndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}
	if req.TimeType == 0 {
		return getProviderServiceStatusByDay(req, reply)
	}
	return getProviderServiceStatusByHour(req, reply)
}
