// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func getProviderTodayData(req *apis.BiGetProviderTrendReq) (*apis.Trend, error) {
	mreq := apis.LogQueryProviderCountExportReq{}
	if len(req.ProviderApiId) != 0 {
		mreq.ProviderApiIdList = append(mreq.ProviderApiIdList, req.ProviderApiId...)
	}
	mreq.TabName = "t_gd_thirdpart_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = getTodayTimestamp()
	if req.TodayEndTimestamp > 0 && req.TodayEndTimestamp < mreq.EndTimestamp {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryProviderCountExportReply{}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := apis.Trend{
		Date: time.Unix(mreq.StartTimestamp, 0).Format("2006-01-02"),
	}

	for _, v := range mreply.LogQueryProviderCount {
		ret.Total += v.Total
		ret.Charge += v.Charge
	}
	return &ret, nil
}

func getProviderTodayDataByHour(req *apis.BiGetProviderTrendReq, start, end int64) (*apis.HourTrend, error) {
	mreq := apis.LogQueryProviderCountExportReq{}
	if len(req.ProviderApiId) != 0 {
		mreq.ProviderApiIdList = append(mreq.ProviderApiIdList, req.ProviderApiId...)
	}
	mreq.TabName = "t_gd_thirdpart_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = start, end
	if req.TodayEndTimestamp > 0 && mreq.EndTimestamp > req.TodayEndTimestamp {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryProviderCountExportReply{}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := apis.HourTrend{
		Hour: time.Unix(mreq.StartTimestamp, 0).Format("2006-01-02 15:04:05"),
	}

	for _, v := range mreply.LogQueryProviderCount {
		ret.Total += v.Total
		ret.Charge += v.Charge
	}
	return &ret, nil
}

func getTrendFromProviderReportByHour(startTimestamp, endTimestamp, lastHourTimestamp int64, req *apis.BiGetProviderTrendReq) ([]apis.HourTrend, int, error) {

	o := orm.NewOrm()
	array, hours := getHours(startTimestamp, endTimestamp, lastHourTimestamp)
	sql := fmt.Sprintf("select from_unixtime(hour) as hour, sum(total) as total, sum(charge) as charge from t_gd_provider_report_hour where hour in(%s)", hours)

	if len(req.ProviderApiIdNew) != 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by hour order by hour", sql)
	trends := []apis.HourTrend{}
	if hours == "" {
		return trends, len(array), nil
	}
	_, err := o.Raw(sql).QueryRows(&trends)
	if err != nil && err != orm.ErrNoRows {
		return nil, 0, errors.DataBaseError
	}
	return trends, len(array), nil
}

func getTrendFromProviderReport(startTimestamp, endTimestamp int64, req *apis.BiGetProviderTrendReq) ([]apis.Trend, int, error) {
	o := orm.NewOrm()
	array, dates := getDates(startTimestamp, endTimestamp)
	sql := fmt.Sprintf("select date, sum(total) as total, sum(charge) as charge from t_gd_provider_report where date in(%s)", dates)

	if len(req.ProviderApiIdNew) != 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by date order by date", sql)
	trends := []apis.Trend{}
	_, err := o.Raw(sql).QueryRows(&trends)
	if err != nil && err != orm.ErrNoRows {
		return nil, 0, errors.DataBaseError
	}
	return trends, len(array), nil
}

func getProviderLastStageAvgByHour(req *apis.BiGetProviderTrendReq, lastHourTimestamp int64, lastHourTrend *apis.HourTrend, hours int) (int64, int64, error) {
	startTimestamp, endTimestamp := getLastStageOfHourTimestamp(req.StartTimestamp, hours)
	trends, _, err := getTrendFromProviderReportByHour(startTimestamp, endTimestamp, lastHourTimestamp, req)
	if len(trends) == 0 {
		return 0, 0, nil
	}
	if err != nil {
		return 0, 0, err
	}
	if trends == nil {
		trends = []apis.HourTrend{}
	}
	if lastHourTimestamp < endTimestamp {
		trends = append(trends, *lastHourTrend)
	}
	total := 0
	charge := 0
	for _, v := range trends {
		total += int(v.Total)
		charge += int(v.Charge)
	}
	return int64(total / (len(trends))), int64(charge / (len(trends))), nil
}

func getProviderLastStageAvg(req *apis.BiGetProviderTrendReq, days int) (int64, int64, error) {
	startTimestamp, endTimestamp := getLastStageTimestamp(req.StartTimestamp, days)
	trends, _, err := getTrendFromProviderReport(startTimestamp, endTimestamp, req)
	if len(trends) == 0 {
		return 0, 0, nil
	}
	if err != nil {
		return 0, 0, err
	}
	total := 0
	charge := 0
	for _, v := range trends {
		total += int(v.Total)
		charge += int(v.Charge)
	}
	return int64(total / (len(trends))), int64(charge / (len(trends))), nil
}

func BiGetProviderTrendByDay(req *apis.BiGetProviderTrendReq, reply *apis.BiGetProviderTrendReply) (err error) {
	newid := req.ProviderApiId
	req.ProviderApiIdNew = newid
	trends, days, err := getTrendFromProviderReport(req.StartTimestamp, req.EndTimestamp, req)
	if err != nil {
		return err
	}
	now := time.Now()
	today := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
	nowDate := time.Now().Format("2006-01-02")
	todayTrend, err := getProviderTodayData(req)
	if err != nil {
		return err
	}
	if today >= req.StartTimestamp && today < req.EndTimestamp {
		exist := false
		for _, v := range trends {
			if v.Date == nowDate {
				exist = true
				break
			}
		}
		if exist == false {
			trends = append(trends, *todayTrend)
		}
	}
	if reply.LastStageTotalAvg, reply.LastStageChargeAvg, err = getProviderLastStageAvg(req, days); err != nil {
		return err
	}
	maxTotal := int64(0)
	maxCharge := int64(0)
	minCharge := int64(0)
	minTotal := int64(0)
	sumTotal := int64(0)
	sumCharge := int64(0)
	for i, v := range trends {
		if i == 0 {
			minTotal = v.Total
			minCharge = v.Charge
		}
		if v.Total > maxTotal {
			maxTotal = v.Total
		}
		if v.Total < minTotal {
			minTotal = v.Total
		}
		if v.Charge > maxCharge {
			maxCharge = v.Charge
		}
		if v.Charge < minCharge {
			minCharge = v.Charge
		}

		sumTotal += v.Total
		sumCharge += v.Charge
	}

	m := map[string]*apis.Trend{}
	for i, v := range trends {
		m[v.Date] = &trends[i]
	}
	for i := req.StartTimestamp; i < req.EndTimestamp; i = i + 24*3600 {
		date := time.Unix(i, 0).Format("2006-01-02")
		if p, ok := m[date]; ok {
			p.Date = parseTrendDate(p.Date)
			reply.Infos = append(reply.Infos, *p)
		} else {
			info := apis.Trend{
				Date: parseTrendDate(date),
			}
			reply.Infos = append(reply.Infos, info)
		}
	}
	if len(reply.Infos) == 0 {
		return nil
	}
	reply.ChargeMax = maxCharge
	reply.TotalMax = maxTotal
	reply.TotalMin = minTotal
	reply.ChargeMin = minCharge
	reply.ChargeCurrent = todayTrend.Charge
	reply.TotalCurrent = todayTrend.Total
	reply.ChargeAvg = int64(sumCharge / int64(len(reply.Infos)))
	reply.TotalAvg = int64(sumTotal / int64(len(reply.Infos)))
	return nil
}

func BiGetProviderTrendByHour(req *apis.BiGetProviderTrendReq, reply *apis.BiGetProviderTrendReply) (err error) {
	req.ProviderApiIdNew = req.ProviderApiId

	start, end := getCurrentHour()
	if req.TodayEndTimestamp > 0 && req.TodayEndTimestamp < req.EndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}

	trends, hours, err := getTrendFromProviderReportByHour(req.StartTimestamp, req.EndTimestamp, start-3600, req)
	if err != nil {
		return err
	}
	currentHourTrend, err := getProviderTodayDataByHour(req, start, end)
	if err != nil {
		return err
	}
	lastHourTrend, err := getProviderTodayDataByHour(req, start-3600, end-3600)
	if err != nil {
		return err
	}

	if req.EndTimestamp > start {
		trends = append(trends, *currentHourTrend)
	}
	if req.EndTimestamp > start-3600 {
		trends = append(trends, *lastHourTrend)
	}

	if reply.LastStageTotalAvg, reply.LastStageChargeAvg, err = getProviderLastStageAvgByHour(req, start-3600, lastHourTrend, hours); err != nil {
		return err
	}
	maxTotal := int64(0)
	maxCharge := int64(0)
	minTotal := int64(0)
	minCharge := int64(0)
	sumTotal := int64(0)
	sumCharge := int64(0)
	for i, v := range trends {
		if i == 0 {
			minTotal = v.Total
			minCharge = v.Charge
		}
		if v.Total > maxTotal {
			maxTotal = v.Total
		}
		if v.Total < minTotal {
			minTotal = v.Total
		}
		if v.Charge > maxCharge {
			maxCharge = v.Charge
		}
		if v.Charge < minCharge {
			minCharge = v.Charge
		}
		sumTotal += v.Total
		sumCharge += v.Charge
	}

	m := map[string]*apis.HourTrend{}
	for i, v := range trends {
		m[v.Hour] = &trends[i]
	}
	for i := req.StartTimestamp; i < req.EndTimestamp; i = i + 3600 {
		hour := time.Unix(i, 0).Format("2006-01-02 15:04:05")
		if p, ok := m[hour]; ok {
			item := apis.Trend{}
			item.Date = parseTrendHour(p.Hour)
			item.Total = p.Total
			item.Charge = p.Charge
			reply.Infos = append(reply.Infos, item)
		} else {
			info := apis.Trend{
				Date: parseTrendHour(hour),
			}
			reply.Infos = append(reply.Infos, info)
		}
	}
	if len(reply.Infos) == 0 {
		return nil
	}
	reply.ChargeMax = maxCharge
	reply.TotalMax = maxTotal
	reply.TotalMin = minTotal
	reply.ChargeMin = minCharge
	reply.ChargeCurrent = currentHourTrend.Charge
	reply.TotalCurrent = currentHourTrend.Total
	reply.ChargeAvg = int64(sumCharge / int64(len(reply.Infos)))
	reply.TotalAvg = int64(sumTotal / int64(len(reply.Infos)))
	return nil
}

func BiGetProviderTrend(ctx context.Context, req *apis.BiGetProviderTrendReq, reply *apis.BiGetProviderTrendReply) (err error) {
	if req.TimeType == 0 {
		return BiGetProviderTrendByDay(req, reply)
	}
	return BiGetProviderTrendByHour(req, reply)
}
