// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

type TGdReport struct {
	Id                int64
	MerchantId        int64
	ApiId             int64
	MerchantName      string
	Date              string `json:"date"`
	MerchantDataApiId int64
	ComboType         int
	Count             int
	Used              int
	Remain            int
	ApiName           string `json:"api_name"`
	Total             int64  `json:"total"`
	Valid             int64  `json:"valid"`
	Success           int64  `json:"success"`
	Query             int64  `json:"query"`
	Reuse             int64  `json:"reuse"`
	ValidRate         string `json:"valid_rate"`
	SuccessRate       string `json:"success_rate"`
	QueryRate         string `json:"query_rate"`
	ReuseRate         string `json:"reuse_rate"`
	AvgElapsed        string `json:"avg_elapsed"`
	PlatformError     int64  `json:"platform_error"`
	PlatformErrorRate string `json:"platform_error_rate"`
	ProviderError     int64  `json:"provider_error"`
	ProviderErrorRate string `json:"provider_error_rate"`
	BigElapsedList    string `json:"big_elapsed_list"`
	Charge            int64  `json:"charge"`
	ChargeRate        string `json:"charge_rate"`
	TimeoutInfo       string `json:"timeout_info"`
	TimeoutCount      int    `json:"timeout_count"`
	Repeated          bool   `json:"repeated"`
}

func getTodayReport(req *apis.BiGetReportReq) ([]apis.BiReportInfo, error) {
	mreq := apis.LogQueryUserAccessCountExportReq{}
	mreq.MerchantId = req.MerchantId
	mreq.ApiIdList = req.ApiId
	mreq.GroupByMerchant = true
	mreq.TabName = "t_gd_access_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = getTodayTimestamp()
	if req.TodayEndTimestamp > 0 {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryUserAccessCountExportReply{}
	err := LogQueryUserAccessCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := make([]apis.BiReportInfo, len(mreply.LogQueryUserAcessCount))
	for i, v := range mreply.LogQueryUserAcessCount {
		item := apis.BiReportInfo{}
		item.Date = time.Now().Format("2006-01-02")
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.Valid = v.Valid
		item.ValidRate = v.ValidRate
		item.AvgElapsed = strconv.FormatFloat(v.AvgElapsed, 'f', 2, 64)
		item.Fail = v.Valid - v.Success
		item.Success = v.Success
		item.ApiName = v.ApiName
		item.MerchantName = v.MerchantName
		item.QueryNoRecord = v.Query
		item.Reuse = v.Reuse
		item.ApiId = v.ApiId
		if item.Valid == 0 {
			item.FailRate = "0%"
			item.ReuseRate = "0%"
			item.QueryNoRecordRate = "0%"
		} else {
			item.FailRate = strconv.FormatFloat(float64(100*item.Fail)/float64(item.Valid), 'f', 2, 64) + "%"
			item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ReuseRate = strconv.FormatFloat(float64(100*item.Reuse/item.Valid), 'f', 2, 64) + "%"
		}
		ret[i] = item
	}
	return ret, nil
}

func getTodayReportFromHour(req *apis.BiGetReportReq, start, end int64) ([]apis.BiReportInfo, error) {
	if req.TodayEndTimestamp > 0 && req.TodayEndTimestamp < end {
		end = req.TodayEndTimestamp
	}
	ret := []apis.BiReportInfo{}
	sql := fmt.Sprintf("select merchant_name, api_name, api_id, sum(charge) as charge, sum(total) as total,sum(valid) as valid,"+
		"sum(sum_elapsed) as sum_elapsed, sum(query) as query_no_record, sum(reuse)  as reuse, sum(success) as success from t_gd_report_hour "+
		"where hour >= %d and hour < %d", start, end)
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", req.MerchantId)
	}
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by merchant_id, api_id", sql)
	if _, err := orm.NewOrm().Raw(sql).QueryRows(&ret); err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	for i, v := range ret {
		ret[i].Date = time.Now().Format("2006-01-02")
		ret[i].ChargeRate = strconv.FormatFloat(float64(100*v.Charge)/float64(v.Total), 'f', 2, 64) + "%"
		ret[i].ValidRate = strconv.FormatFloat(float64(100*v.Valid)/float64(v.Total), 'f', 2, 64) + "%"
		ret[i].AvgElapsed = strconv.FormatFloat(v.SumElapsed/float64(v.Total), 'f', 2, 64)
		ret[i].Fail = v.Valid - v.Success
		if v.Valid == 0 {
			ret[i].FailRate = "0%"
			ret[i].ReuseRate = "0%"
			ret[i].QueryNoRecordRate = "0%"
		} else {
			ret[i].FailRate = strconv.FormatFloat(float64(100*v.Fail)/float64(v.Valid), 'f', 2, 64) + "%"
			ret[i].QueryNoRecordRate = strconv.FormatFloat(float64(100*v.QueryNoRecord)/float64(v.Valid), 'f', 2, 64) + "%"
			ret[i].ReuseRate = strconv.FormatFloat(float64(100*v.Reuse/v.Valid), 'f', 2, 64) + "%"
		}
	}
	return ret, nil
}

var pageSize = 10

func parseDate(dateArray []string) (dates string, containToday bool) {
	nowday := time.Now().Format("2006-01-02")
	for _, v := range dateArray {
		if dates == "" {
			dates = fmt.Sprintf("'%s'", v)
		} else {
			dates = fmt.Sprintf("%s,'%s'", dates, v)
		}
		if v == nowday {
			containToday = true
		}
	}
	return dates, containToday
}

func parseToday(req *apis.BiGetReportReq, reply *apis.BiGetReportReply) error {
	infos, err := getTodayReport(req)
	if err != nil {
		return err
	}
	reply.Infos = infos
	return nil
}

func parseTodayByApi(req *apis.BiGetReportReq, reply *apis.BiGetReportReply) error {
	infos, err := getTodayReport(req)
	if err != nil {
		return err
	}
	reply.Infos = infos
	return nil
}

func parseOldDay(o orm.Ormer, req *apis.BiGetReportReq, reply *apis.BiGetReportReply, dates string) error {
	tabs := []TGdReport{}

	// sql prepare
	sql := fmt.Sprintf("select * from t_gd_report where repeated = 0 and date in(%s) ", dates)
	if len(req.ApiId) != 0 {
		ids := ""
		for _, v := range req.ApiId {
			if ids == "" {
				ids = fmt.Sprintf("%d", v)
			} else {
				ids = fmt.Sprintf("%s,%d", ids, v)
			}
		}
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, ids)
	}
	if len(req.MerchantIds) > 0 {
		sub := "merchant_id in (" + strings.Replace(strings.Trim(fmt.Sprint(req.MerchantIds), "[]"), " ", ",", -1) + ")"
		sql = fmt.Sprintf("%s and %s", sql, sub)
	} else if req.MerchantId != 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	sql = fmt.Sprintf("%s order by date, merchant_id, api_id", sql)

	// exec sql
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	//parse result
	infos := make([]apis.BiReportInfo, len(tabs))
	for i, v := range tabs {
		item := apis.BiReportInfo{}
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.Valid = v.Valid
		item.ValidRate = v.ValidRate
		item.AvgElapsed = v.AvgElapsed
		item.Fail = v.Valid - v.Success
		item.ApiName = v.ApiName
		item.MerchantName = v.MerchantName
		item.Success = v.Success
		item.QueryNoRecord = v.Query
		item.Reuse = v.Reuse

		if item.Valid == 0 {
			item.FailRate = "0%"
			item.ReuseRate = "0%"
			item.QueryNoRecordRate = "0%"
		} else {
			item.FailRate = strconv.FormatFloat(float64(100*item.Fail)/float64(item.Valid), 'f', 2, 64) + "%"
			item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ReuseRate = strconv.FormatFloat(float64(100*item.Reuse/item.Valid), 'f', 2, 64) + "%"
		}
		infos[i] = item
	}
	reply.Infos = append(reply.Infos, infos...)
	return nil
}

func parseOldDayByApi(o orm.Ormer, req *apis.BiGetReportReq, reply *apis.BiGetReportReply, dates string) error {
	tabs := []TGdReport{}

	// sql prepare
	sql := fmt.Sprintf("select * from t_gd_report where repeated = 0 and date in(%s) ", dates)
	if len(req.ApiId) != 0 {
		ids := ""
		for _, v := range req.ApiId {
			if ids == "" {
				ids = fmt.Sprintf("%d", v)
			} else {
				ids = fmt.Sprintf("%s,%d", ids, v)
			}
		}
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, ids)
	}
	if req.MerchantId != 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	sql = fmt.Sprintf("%s order by date, merchant_id, api_id", sql)

	// exec sql
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	//parse result
	infos := make([]apis.BiReportInfo, len(tabs))
	for i, v := range tabs {
		item := apis.BiReportInfo{}
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = v.ChargeRate
		item.Total = v.Total
		item.Valid = v.Valid
		item.ValidRate = v.ValidRate
		item.AvgElapsed = v.AvgElapsed
		item.Fail = v.Valid - v.Success
		item.ApiName = v.ApiName
		item.MerchantName = v.MerchantName
		item.Success = v.Success
		item.QueryNoRecord = v.Query
		item.Reuse = v.Reuse
		item.ApiId = v.ApiId

		if item.Valid == 0 {
			item.FailRate = "0%"
			item.ReuseRate = "0%"
			item.QueryNoRecordRate = "0%"
		} else {
			item.FailRate = strconv.FormatFloat(float64(100*item.Fail)/float64(item.Valid), 'f', 2, 64) + "%"
			item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ReuseRate = strconv.FormatFloat(float64(100*item.Reuse/item.Valid), 'f', 2, 64) + "%"
		}
		infos[i] = item
	}
	reply.Infos = append(reply.Infos, infos...)
	return nil
}

func sortReport(array []apis.BiReportInfo) []apis.BiReportInfo {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Total < array[j].Total {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	return array

}

func sumReport(infos []apis.BiReportInfo) apis.BiReportInfo {
	ret := apis.BiReportInfo{}
	sumElapsed := float64(0)
	if len(infos) == 0 {
		return ret
	}
	for _, v := range infos {
		ret.Total += v.Total
		ret.Charge += v.Charge
		ret.Success += v.Success
		ret.Reuse += v.Reuse
		ret.Valid += v.Valid
		ret.QueryNoRecord += v.QueryNoRecord
		ret.Fail += v.Fail
		avgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		sumElapsed += (avgElapsed * float64(v.Total))
	}
	ret.AvgElapsed = strconv.FormatFloat(sumElapsed/(float64(ret.Total)), 'f', 2, 64)
	ret.ChargeRate = strconv.FormatFloat(float64(100*ret.Charge)/float64(ret.Total), 'f', 2, 64) + "%"
	ret.ValidRate = strconv.FormatFloat(float64(100*ret.Valid)/float64(ret.Total), 'f', 2, 64) + "%"
	if ret.Valid == 0 {
		ret.FailRate = "0%"
		ret.ReuseRate = "0%"
		ret.QueryNoRecordRate = "0%"
	} else {
		ret.FailRate = strconv.FormatFloat(float64(100*ret.Fail)/float64(ret.Valid), 'f', 2, 64) + "%"
		ret.QueryNoRecordRate = strconv.FormatFloat(float64(100*ret.QueryNoRecord)/float64(ret.Valid), 'f', 2, 64) + "%"
		ret.ReuseRate = strconv.FormatFloat(float64(100*ret.Reuse)/float64(ret.Valid), 'f', 2, 64) + "%"
	}
	return ret
}

func BiGetReport(ctx context.Context, req *apis.BiGetReportReq, reply *apis.BiGetReportReply) (err error) {
	// prepare
	if req.PageSize == 0 {
		req.PageSize = pageSize
	}
	reply.PageSize = req.PageSize
	if len(req.Date) == 0 {
		return errors.ArgsError
	}
	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	offset := (req.PageNumber - 1) * req.PageSize
	limit := req.PageSize

	// parse date to sql string
	dates, containToday := parseDate(req.Date)

	// get total count
	o := orm.NewOrm()
	if containToday {
		err = parseToday(req, reply)
		if err != nil {
			return err
		}
	}
	if err := parseOldDay(o, req, reply, dates); err != nil {
		return err
	}
	reply.Infos = sortReport(reply.Infos)
	reply.Sum = sumReport(reply.Infos)
	reply.Total = len(reply.Infos)

	// 直接返回所有数据由前端排序
	if req.Page == false {
		return nil
	}

	if req.IsAll {
		return nil
	}
	switch {
	case reply.Total-offset <= 0:
		reply.Infos = []apis.BiReportInfo{}
	case reply.Total-offset <= limit:
		reply.Infos = reply.Infos[offset:]
	case reply.Total-offset > limit:
		reply.Infos = reply.Infos[offset : offset+limit]
	}
	return nil
}

func BiGetReportByApi(ctx context.Context, req *apis.BiGetReportReq, reply *apis.BiGetReportReply) (err error) {
	// prepare
	reply.PageSize = pageSize
	if len(req.Date) == 0 {
		return errors.ArgsError
	}
	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	offset := (req.PageNumber - 1) * pageSize
	limit := pageSize

	// parse date to sql string
	dates, containToday := parseDate(req.Date)

	// get total count
	o := orm.NewOrm()
	if containToday {
		err = parseTodayByApi(req, reply)
		if err != nil {
			return err
		}
	}
	parseOldDayByApi(o, req, reply, dates)

	m := map[int64]*apis.BiReportInfo{}
	oldSumElapsed := float64(0)
	for i, v := range reply.Infos {
		key := v.ApiId
		if _, ok := m[key]; ok == false {
			m[key] = &reply.Infos[i]
			m[key].MerchantName = ""
			continue
		}
		//处理平均时间
		newAvgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		newSumElapsed := newAvgElapsed * float64(v.Total)
		if oldSumElapsed == 0 {
			oldAvgElapsed, _ := strconv.ParseFloat(m[key].AvgElapsed, 64)
			oldSumElapsed = oldAvgElapsed * float64(m[key].Total)
		}
		m[key].AvgElapsed = fmt.Sprintf("%f", (newSumElapsed+oldSumElapsed)/float64(m[key].Total+v.Total))
		oldSumElapsed = newSumElapsed + oldSumElapsed

		// 部分指标累加
		m[key].Total += v.Total
		m[key].Charge += v.Charge
		m[key].Valid += v.Valid
		m[key].QueryNoRecord += v.QueryNoRecord
		m[key].Success += v.Success
		m[key].Reuse += v.Reuse

	}

	//parse result
	infos := make([]apis.BiReportInfo, pageSize)

	i := 0
	count := 0
	for _, v := range m {
		if i < offset {
			i++
			continue
		}
		if count >= limit {
			break
		}
		item := apis.BiReportInfo{}
		item.Total = v.Total
		item.Date = v.Date
		item.Charge = v.Charge
		item.ChargeRate = strconv.FormatFloat(float64(100*item.Charge)/float64(item.Total), 'f', 2, 64) + "%"
		item.Valid = v.Valid
		item.ValidRate = strconv.FormatFloat(float64(100*item.Valid)/float64(item.Total), 'f', 2, 64) + "%"
		avgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		item.AvgElapsed = strconv.FormatFloat(avgElapsed, 'f', 2, 64)
		item.Fail = v.Fail
		item.Success = v.Success
		item.ApiName = v.ApiName
		item.MerchantName = v.MerchantName
		item.QueryNoRecord = v.QueryNoRecord
		item.Reuse = v.Reuse
		if item.Valid == 0 {
			item.FailRate = "0%"
			item.ReuseRate = "0%"
			item.QueryNoRecordRate = "0%"
		} else {
			item.FailRate = strconv.FormatFloat(float64(100*item.Fail)/float64(item.Valid), 'f', 2, 64) + "%"
			item.QueryNoRecordRate = strconv.FormatFloat(float64(100*item.QueryNoRecord)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ReuseRate = strconv.FormatFloat(float64(100*item.Reuse/item.Valid), 'f', 2, 64) + "%"
		}
		infos[count] = item
		i++
		count++
	}
	reply.Infos = infos[:count]
	reply.Total = len(m)
	return nil
}
