// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

type ServiceStatus struct {
	FailCount       int64
	SumElapsed      float64
	Total           int64
	TimestampMinute int64
}

func getServiceStatusFromReport(req *apis.BiGetServiceStatusReq, dates string, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if dates == "" {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(avg_elapsed * total) as sum_elapsed, sum(total) as count, sum(valid - success) as fail_count, date from t_gd_report where date in(%s)", dates)
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by date", sql)
	_, err := o.Raw(sql).QueryRows(&ret)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func getServiceStatusFromHourReport(req *apis.BiGetServiceStatusReq, hours string, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if hours == "" {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(sum_elapsed) as sum_elapsed, sum(total) as count, sum(valid - success) as fail_count, FROM_UNIXTIME(hour) as date from t_gd_report_hour where hour in(%s)", hours)

	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by hour", sql)
	_, err := o.Raw(sql).QueryRows(&ret)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func getServiceStatusFromDayLog(req *apis.BiGetServiceStatusReq, start, end int64, o orm.Ormer, isHour bool) (*apis.ServiceStatus, error) {
	ret := &apis.ServiceStatus{}

	sql := fmt.Sprintf("select sum(elapsed) as sum_elapsed, sum(1) as count, (sum(code=0 or code=1100 or code=1101 or code=1000 or code=1001) - sum(code=0 or code=1100)) as fail_count from t_gd_access_log_day where timestamp >= %d and timestamp < %d", start, end)

	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	err := o.Raw(sql).QueryRow(ret)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	ret.Date = time.Unix(start, 0).Format("2006-01-02")
	if isHour {
		ret.Date = time.Unix(start, 0).Format("2006-01-02 15:04:05")
	}
	return ret, nil
}

func getServiceStatusFromDayLogByHour(req *apis.BiGetServiceStatusReq, start, end int64, o orm.Ormer) ([]apis.ServiceStatus, error) {
	ret := []apis.ServiceStatus{}
	if start == 0 || end == 0 {
		return ret, nil
	}
	for i := start; i < end; i += 3600 {
		e := start + 3600
		if i+3600 > e {
			e = end
		}
		info, err := getServiceStatusFromDayLog(req, i, e, o, true)
		if err != nil {
			return nil, err
		}
		ret = append(ret, *info)
	}
	return ret, nil
}

func combineServiceStatusDay(old []apis.ServiceStatus, today *apis.ServiceStatus, dateArray []string) []apis.ServiceStatus {
	m := map[string]*apis.ServiceStatus{}
	for i, v := range old {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &old[i]
		}
	}
	if today.Date != "" {
		m[today.Date] = today
	}
	ret := make([]apis.ServiceStatus, len(dateArray))
	for i, v := range dateArray {
		if _, ok := m[v]; ok == false {
			item := apis.ServiceStatus{}
			item.TimestampMinute = DateToTimestamp(v)
			ret[i] = item
			continue
		}
		item := apis.ServiceStatus{}
		item.TimestampMinute = DateToTimestamp(v)
		item.FailCount = m[v].FailCount
		if m[v].Count > 0 {
			item.AvgElapsed = m[v].SumElapsed / m[v].Count
		} else {
			item.AvgElapsed = 0
		}
		ret[i] = item
	}
	return ret
}

func HourToTimestamp(date string) int64 {
	timeLayout := "2006-01-02 15:04:05"
	loc, _ := time.LoadLocation("Local")
	t, _ := time.ParseInLocation(timeLayout, date, loc)
	return t.Unix()
}

func combineServiceStatusHour(old []apis.ServiceStatus, newInfo []apis.ServiceStatus, hourArray []string) []apis.ServiceStatus {
	m := map[string]*apis.ServiceStatus{}
	for i, v := range old {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &old[i]
		}
	}
	for i, v := range newInfo {
		if _, ok := m[v.Date]; ok == false {
			m[v.Date] = &newInfo[i]
		}
	}

	ret := make([]apis.ServiceStatus, len(hourArray))
	for i, v := range hourArray {
		if _, ok := m[v]; ok == false {
			item := apis.ServiceStatus{}
			item.TimestampMinute = HourToTimestamp(v)
			item.FailCount = 0
			item.AvgElapsed = 0.0
			ret[i] = item
			continue
		}
		item := apis.ServiceStatus{}
		item.TimestampMinute = HourToTimestamp(v)
		item.FailCount = m[v].FailCount
		if m[v].Count > 0 {
			item.AvgElapsed = m[v].SumElapsed / m[v].Count
		} else {
			item.AvgElapsed = 0
		}
		ret[i] = item
	}
	return ret
}

func getServiceStatusByDay(req *apis.BiGetServiceStatusReq, reply *apis.BiGetServiceStatusReply) error {
	dateArray := getDatesFromTimestamp(req.StartTimestamp, req.EndTimestamp)
	now := time.Now()
	dates := ""
	todayZero := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
	o := orm.NewOrm()
	containToday := false
	for _, v := range dateArray {
		if v == now.Format("2006-01-02") {
			containToday = true
			continue
		}
		if dates == "" {
			dates = fmt.Sprintf("'%s'", v)
			continue
		}
		dates = fmt.Sprintf("%s, '%s'", dates, v)
	}
	oldInfo, err := getServiceStatusFromReport(req, dates, o)
	if err != nil {
		return err
	}
	todayInfo := &apis.ServiceStatus{}
	if containToday {
		todayInfo, err = getServiceStatusFromDayLog(req, todayZero.Unix(), req.EndTimestamp, o, false)
		if err != nil {
			return err
		}
	}
	reply.Infos = combineServiceStatusDay(oldInfo, todayInfo, dateArray)
	return nil
}

func getHoursByTimestamp(startTimestamp int64, endTimestamp int64) ([]string, string) {
	ret := []string{}
	retString := ""
	if startTimestamp == 0 || endTimestamp == 0 {
		return nil, ""
	}
	for i := startTimestamp; i <= endTimestamp; i = i + 3600 {
		date := time.Unix(i, 0).Format("2006-01-02 15:04:05")
		ret = append(ret, date)
		if retString == "" {
			retString = fmt.Sprintf("%d", i)
		} else {
			retString = fmt.Sprintf("%s,%d", retString, i)
		}
	}
	return ret, retString
}
func getServiceStatusByHour(req *apis.BiGetServiceStatusReq, reply *apis.BiGetServiceStatusReply) error {
	o := orm.NewOrm()
	tabs := []string{
		"t_gd_report_hour",
	}
	hourStart, hourEnd, otherStart, otherEnd, err := parseHourTimestamp(o, req.StartTimestamp, req.EndTimestamp, tabs)
	if err != nil {
		return err
	}
	_, hours := getHoursByTimestamp(hourStart, hourEnd)
	hourArray, _ := getHoursByTimestamp(req.StartTimestamp, req.EndTimestamp)
	oldInfo, err := getServiceStatusFromHourReport(req, hours, o)
	if err != nil {
		return err
	}
	newInfo, err := getServiceStatusFromDayLogByHour(req, otherStart, otherEnd, o)
	if err != nil {
		return err
	}
	reply.Infos = combineServiceStatusHour(oldInfo, newInfo, hourArray)
	return nil
}

func BiGetServiceStatus(ctx context.Context, req *apis.BiGetServiceStatusReq, reply *apis.BiGetServiceStatusReply) error {
	if req.TodayEndTimestamp > 0 && req.EndTimestamp > req.TodayEndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}
	if req.TimeType == 0 {
		return getServiceStatusByDay(req, reply)
	}
	return getServiceStatusByHour(req, reply)
}
