// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"strings"
	"time"
)

func getTodayTimestamp() (int64, int64) {
	now := time.Now()
	start := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
	next := now.AddDate(0, 0, 1)
	end := time.Date(next.Year(), next.Month(), next.Day(), 0, 0, 0, 0, next.Location()).Unix()
	return start, end
}
func getTodayData(req *apis.BiGetTrendReq) (*apis.Trend, error) {
	mreq := apis.LogQueryUserAccessCountExportReq{}
	mreq.MerchantId = req.MerchantId
	if len(req.ApiId) != 0 {
		mreq.ApiIdList = append(mreq.ApiIdList, req.ApiId...)
	}
	mreq.GroupByMerchant = true
	mreq.TabName = "t_gd_access_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = getTodayTimestamp()
	if req.TodayEndTimestamp > 0 {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryUserAccessCountExportReply{}
	err := LogQueryUserAccessCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := apis.Trend{
		Date: time.Unix(mreq.StartTimestamp, 0).Format("2006-01-02"),
	}

	for _, v := range mreply.LogQueryUserAcessCount {
		ret.Total += v.Valid
		ret.Charge += v.Charge
	}
	return &ret, nil
}

func getTodayDataByHour(req *apis.BiGetTrendReq, start, end int64) (*apis.HourTrend, error) {
	mreq := apis.LogQueryUserAccessCountExportReq{}
	mreq.MerchantId = req.MerchantId
	if len(req.ApiId) != 0 {
		mreq.ApiIdList = append(mreq.ApiIdList, req.ApiId...)
	}
	mreq.GroupByMerchant = true
	mreq.TabName = "t_gd_access_log_day"
	mreq.StartTimestamp, mreq.EndTimestamp = start, end

	if req.TodayEndTimestamp > 0 && mreq.EndTimestamp > req.TodayEndTimestamp {
		mreq.EndTimestamp = req.TodayEndTimestamp
	}
	mreply := apis.LogQueryUserAccessCountExportReply{}
	err := LogQueryUserAccessCountExport(context.Background(), &mreq, &mreply)
	if err != nil {
		return nil, err
	}
	ret := apis.HourTrend{
		Hour: time.Unix(mreq.StartTimestamp, 0).Format("2006-01-02 15:04:05"),
	}

	for _, v := range mreply.LogQueryUserAcessCount {
		ret.Total += v.Valid
		ret.Charge += v.Charge
	}
	return &ret, nil
}

func getDates(startTimestamp int64, endTimestamp int64) ([]string, string) {
	ret := []string{}
	retString := ""
	for i := startTimestamp; i <= endTimestamp; i = i + 24*3600 {
		date := time.Unix(i, 0).Format("2006-01-02")
		ret = append(ret, date)
		if retString == "" {
			retString = fmt.Sprintf("'%s'", date)
		} else {
			retString = fmt.Sprintf("%s,'%s'", retString, date)
		}
	}
	return ret, retString
}

func getHours(startTimestamp int64, endTimestamp int64, lastHourTimestamp int64) ([]string, string) {
	ret := []string{}
	retString := ""
	for i := startTimestamp; i <= endTimestamp; i = i + 3600 {
		date := time.Unix(i, 0).Format("2006-01-02 15:04:05")
		ret = append(ret, date)
		if i >= lastHourTimestamp {
			continue
		}
		if retString == "" {
			retString = fmt.Sprintf("%d", i)
		} else {
			retString = fmt.Sprintf("%s,%d", retString, i)
		}
	}
	return ret, retString
}

func getLastStageOfHourTimestamp(startTimestamp int64, hours int) (int64, int64) {
	endTime := time.Unix(startTimestamp, 0)
	return endTime.Unix() - int64(hours*3600), endTime.Unix()
}

func getLastStageTimestamp(startTimestamp int64, days int) (int64, int64) {
	endTime := time.Unix(startTimestamp, 0)
	startTime := endTime.AddDate(0, 0, -days)
	return startTime.Unix(), endTime.Unix()
}

func getTrendFromReport(startTimestamp, endTimestamp int64, req *apis.BiGetTrendReq) ([]apis.Trend, int, error) {
	array, dates := getDates(startTimestamp, endTimestamp)
	sql := fmt.Sprintf("select date, sum(valid) as total, sum(charge) as charge from t_gd_report where repeated = 0 and date in(%s)", dates)
	if req.MerchantId != 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if len(req.ApiId) != 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by date order by date", sql)
	trends := []apis.Trend{}
	o := orm.NewOrm()
	_, err := o.Raw(sql).QueryRows(&trends)
	if err != nil && err != orm.ErrNoRows {
		return nil, 0, errors.DataBaseError
	}
	return trends, len(array), nil
}

func getTrendFromReportHour(startTimestamp, endTimestamp int64, lastHourTimestamp int64, req *apis.BiGetTrendReq) ([]apis.HourTrend, int, error) {
	array, hours := getHours(startTimestamp, endTimestamp, lastHourTimestamp)
	sql := fmt.Sprintf("select from_unixtime(hour) as hour, sum(valid) as total, sum(charge) as charge from t_gd_report_hour where hour in(%s)", hours)
	if req.MerchantId != 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if len(req.ApiId) != 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by hour order by hour", sql)
	trends := []apis.HourTrend{}
	if hours == "" {
		return trends, len(array), nil
	}
	o := orm.NewOrm()
	_, err := o.Raw(sql).QueryRows(&trends)
	if err != nil && err != orm.ErrNoRows {
		return nil, 0, errors.DataBaseError
	}
	return trends, len(array), nil
}

func getLastStageAvgByHour(req *apis.BiGetTrendReq, lastHourTimestamp int64, lastHourTrend *apis.HourTrend, hours int) (int64, int64, error) {
	startTimestamp, endTimestamp := getLastStageOfHourTimestamp(req.StartTimestamp, hours)
	trends, _, err := getTrendFromReportHour(startTimestamp, endTimestamp, lastHourTimestamp, req)
	if err != nil {
		return 0, 0, err
	}
	if trends == nil {
		trends = []apis.HourTrend{}
	}
	if lastHourTimestamp < endTimestamp {
		trends = append(trends, *lastHourTrend)
	}
	if len(trends) == 0 {
		return 0, 0, nil
	}
	total := 0
	charge := 0
	for _, v := range trends {
		total += int(v.Total)
		charge += int(v.Charge)
	}
	return int64(total / (len(trends))), int64(charge / (len(trends))), nil
}

func getLastStageAvg(req *apis.BiGetTrendReq, days int) (int64, int64, error) {
	startTimestamp, endTimestamp := getLastStageTimestamp(req.StartTimestamp, days)
	trends, _, err := getTrendFromReport(startTimestamp, endTimestamp, req)
	if err != nil {
		return 0, 0, err
	}
	if len(trends) == 0 {
		return 0, 0, nil
	}
	total := 0
	charge := 0
	for _, v := range trends {
		total += int(v.Total)
		charge += int(v.Charge)
	}
	return int64(total / (len(trends))), int64(charge / (len(trends))), nil
}

func parseTrendHour(hour string) string {
	array := strings.Split(hour, " ")
	if len(array) == 2 {
		return array[1]
	}
	return hour
}
func parseTrendDate(date string) string {
	array := strings.Split(date, "-")
	if len(array) != 3 {
		return ""
	}
	month := array[1]
	day := array[2]
	if len([]byte(month)) == 2 && []byte(month)[0] == '0' {
		month = fmt.Sprintf("%c", []byte(month)[1])
	}
	if len([]byte(day)) == 2 && []byte(day)[0] == '0' {
		day = fmt.Sprintf("%c", []byte(day)[1])
	}
	return fmt.Sprintf("%s-%s", month, day)
}

func BiGetTrendByDay(req *apis.BiGetTrendReq, reply *apis.BiGetTrendReply) (err error) {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		return errors.ArgsError
	}
	trends, days, err := getTrendFromReport(req.StartTimestamp, req.EndTimestamp, req)
	if err != nil {
		return err
	}
	todayTrend, err := getTodayData(req)
	if err != nil {
		return err
	}
	now := time.Now()
	today := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
	nowDate := time.Now().Format("2006-01-02")
	if today >= req.StartTimestamp && today < req.EndTimestamp {
		exist := false
		for _, v := range trends {
			if v.Date == nowDate {
				exist = true
				break
			}
		}
		if exist == false {
			trends = append(trends, *todayTrend)
		}
	}

	if reply.LastStageTotalAvg, reply.LastStageChargeAvg, err = getLastStageAvg(req, days); err != nil {
		return err
	}
	maxTotal := int64(0)
	maxCharge := int64(0)
	minTotal := int64(0)
	minCharge := int64(0)
	sumTotal := int64(0)
	sumCharge := int64(0)
	for i, v := range trends {
		if i == 0 {
			minTotal = v.Total
			minCharge = v.Charge
		}
		if v.Total > maxTotal {
			maxTotal = v.Total
		}
		if v.Total < minTotal {
			minTotal = v.Total
		}
		if v.Charge > maxCharge {
			maxCharge = v.Charge
		}
		if v.Charge < minCharge {
			minCharge = v.Charge
		}
		sumTotal += v.Total
		sumCharge += v.Charge
	}
	m := map[string]*apis.Trend{}
	for i, v := range trends {
		m[v.Date] = &trends[i]
	}
	for i := req.StartTimestamp; i < req.EndTimestamp; i = i + 24*3600 {
		date := time.Unix(i, 0).Format("2006-01-02")
		if p, ok := m[date]; ok {
			p.Date = parseTrendDate(p.Date)
			reply.Infos = append(reply.Infos, *p)
		} else {
			info := apis.Trend{
				Date: parseTrendDate(date),
			}
			reply.Infos = append(reply.Infos, info)
		}
	}
	if len(reply.Infos) == 0 {
		return nil
	}
	reply.ChargeMax = maxCharge
	reply.TotalMax = maxTotal
	reply.TotalMin = minTotal
	reply.ChargeMin = minCharge
	reply.ChargeCurrent = todayTrend.Charge
	reply.TotalCurrent = todayTrend.Total
	reply.ChargeAvg = int64(sumCharge / int64(len(reply.Infos)))
	reply.TotalAvg = int64(sumTotal / int64(len(reply.Infos)))
	return nil
}

func getCurrentHour() (int64, int64) {
	now := time.Now()
	start := time.Date(now.Year(), now.Month(), now.Day(), now.Hour(), 0, 0, 0, now.Location()).Unix()
	return start, start + 3600
}

func BiGetTrendByHour(req *apis.BiGetTrendReq, reply *apis.BiGetTrendReply) (err error) {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		return errors.ArgsError
	}
	if req.TodayEndTimestamp > 0 && req.TodayEndTimestamp < req.EndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}
	start, end := getCurrentHour()
	trends, hours, err := getTrendFromReportHour(req.StartTimestamp, req.EndTimestamp, start-3600, req)
	if err != nil {
		return err
	}

	currentHourTrend, err := getTodayDataByHour(req, start, end)
	if err != nil {
		return err
	}
	lastHourTrend, err := getTodayDataByHour(req, start-3600, end-3600)
	if err != nil {
		return err
	}

	if req.EndTimestamp > start {
		trends = append(trends, *currentHourTrend)
	}
	if req.EndTimestamp > start-3600 {
		trends = append(trends, *lastHourTrend)
	}

	if reply.LastStageTotalAvg, reply.LastStageChargeAvg, err = getLastStageAvgByHour(req, start-3600, lastHourTrend, hours); err != nil {
		return err
	}
	maxTotal := int64(0)
	maxCharge := int64(0)
	minTotal := int64(0)
	minCharge := int64(0)
	sumTotal := int64(0)
	sumCharge := int64(0)
	for i, v := range trends {
		if i == 0 {
			minTotal = v.Total
			minCharge = v.Charge
		}
		if v.Total > maxTotal {
			maxTotal = v.Total
		}
		if v.Total < minTotal {
			minTotal = v.Total
		}
		if v.Charge > maxCharge {
			maxCharge = v.Charge
		}
		if v.Charge < minCharge {
			minCharge = v.Charge
		}
		sumTotal += v.Total
		sumCharge += v.Charge
	}
	m := map[string]*apis.HourTrend{}
	for i, v := range trends {
		m[v.Hour] = &trends[i]
	}
	for i := req.StartTimestamp; i < req.EndTimestamp; i = i + 3600 {
		hour := time.Unix(i, 0).Format("2006-01-02 15:04:05")
		if p, ok := m[hour]; ok {
			item := apis.Trend{}
			item.Date = parseTrendHour(p.Hour)
			item.Total = p.Total
			item.Charge = p.Charge
			reply.Infos = append(reply.Infos, item)
		} else {
			info := apis.Trend{
				Date: parseTrendHour(hour),
			}
			reply.Infos = append(reply.Infos, info)
		}
	}
	if len(reply.Infos) == 0 {
		return nil
	}
	reply.ChargeMax = maxCharge
	reply.TotalMax = maxTotal
	reply.TotalMin = minTotal
	reply.ChargeMin = minCharge
	reply.ChargeCurrent = currentHourTrend.Charge
	reply.TotalCurrent = currentHourTrend.Total
	reply.ChargeAvg = int64(sumCharge / int64(len(reply.Infos)))
	reply.TotalAvg = int64(sumTotal / int64(len(reply.Infos)))
	return nil
}

func BiGetTrend(ctx context.Context, req *apis.BiGetTrendReq, reply *apis.BiGetTrendReply) (err error) {
	if req.TimeType == 0 {
		return BiGetTrendByDay(req, reply)
	}
	return BiGetTrendByHour(req, reply)
}
