// Copyright 2019 utimes.cc. All rights reserved.
// Use of this source code is governed by utimes.cc.

package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"encoding/json"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func getAccesslogWhereSql(req *apis.LogQueryAccessLogReq, tabName string) string {
	sql := fmt.Sprintf("select * from %s where timestamp >= %d and timestamp <= %d", tabName, req.StartTimestamp, req.EndTimestamp)
	if req.ApiId != 0 {
		sql = fmt.Sprintf("%s and api_id=%d", sql, req.ApiId)
	}
	if req.MerchantDataApiId != 0 {
		sql = fmt.Sprintf("%s and merchant_data_api_id =%d", sql, req.MerchantDataApiId)
	}
	if req.State != 0 {
		realState := 1
		if req.State == 2 {
			realState = 0
		}
		sql = fmt.Sprintf("%s and state=%d", sql, realState)
	}
	if req.MerchantId != 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}

	if req.Code != "" {
		sql = fmt.Sprintf("%s and code = %s", sql, req.Code)
	}
	if req.Search != "" {
		sql = fmt.Sprintf("%s and (search='%s' or raw_search='%s')", sql, req.Search, req.Search)
	} else if req.RequestParam != "" {
		sql = fmt.Sprintf("%s and (request_params like '%s' or raw_response_params like '%s')", sql, "%"+req.RequestParam+"%", "%"+req.RequestParam+"%")
	}
	if req.Max != 0 || req.Min != 0 {
		switch {
		case req.Min != 0 && req.Max != 0:
			sql = fmt.Sprintf("%s and elapsed >= %f and elapsed <= %f", sql, req.Min, req.Max)
		case req.Min != 0:
			sql = fmt.Sprintf("%s and elapsed >= %f", sql, req.Min)
		case req.Max != 0:
			sql = fmt.Sprintf("%s and elapsed <= %f", sql, req.Max)
		}
	}
	return sql

}

func getAccesslogWhere(req *apis.LogQueryAccessLogReq, querySet orm.QuerySeter) orm.QuerySeter {
	cond := orm.NewCondition()
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {

		cond = cond.And("timestamp__gte", req.StartTimestamp).And("timestamp__lte", req.EndTimestamp)
	}
	if req.ApiId != 0 {
		cond = cond.And("api_id", req.ApiId)
	}
	if req.MerchantDataApiId != 0 {
		cond = cond.And("merchant_data_api_id", req.MerchantDataApiId)
	}

	if req.State != 0 {
		realState := 1
		if req.State == 2 {
			realState = 0
		}
		cond = cond.And("state", realState)
	}
	/*if req.PlateNo != "" {
		querySet = querySet.Filter("plate_no", req.PlateNo)
	}
	if req.Vin != "" {
		querySet = querySet.Filter("vin", req.Vin)
	}*/
	if req.MerchantId != 0 {
		cond = cond.And("merchant_id", req.MerchantId)
	}

	if req.Code != "" {
		codeInt, _ := strconv.Atoi(req.Code)
		cond = cond.And("code", codeInt)
	}
	if req.Search != "" {
		cond = cond.And("search", req.Search)
		cond = cond.Or("raw_search", req.Search)
	} else if req.RequestParam != "" {
		cond1 := orm.NewCondition()
		cond = cond.AndCond(cond1.And("request_params__contains", req.RequestParam).Or("raw_response_params__contains", req.RequestParam))
	}

	if req.Max != 0 || req.Min != 0 {
		switch {
		case req.Min != 0 && req.Max != 0:
			cond = cond.And("elapsed__gte", req.Min).And("elapsed__lte", req.Max)
		case req.Min != 0:
			cond = cond.And("elapsed__gte", req.Min)
		case req.Max != 0:
			cond = cond.And("elapsed__lte", req.Max)
		}
	}

	querySet = querySet.SetCond(cond)
	return querySet
}

type MonthTab struct {
	TabName        string
	StartTimestamp int64
	EndTimestamp   int64
	Month          int
	Count          int64
}

func getMonthTabs(prefix string, startTimestamp, endTimestamp int64) []*MonthTab {
	ret := []*MonthTab{}

	startMonth := int(time.Unix(startTimestamp, 0).Month())
	endMonth := int(time.Unix(endTimestamp, 0).Month())
	year := time.Unix(endTimestamp, 0).Year()
	todayStart, todayEnd := getTodayTimestamp()
	if startTimestamp >= todayStart && endTimestamp <= todayEnd {
		item := MonthTab{}
		if strings.Contains(prefix, "third") {
			item.TabName = "t_gd_thirdpart_log_day"
		} else {
			item.TabName = "t_gd_access_log_day"
		}
		item.StartTimestamp = startTimestamp
		item.EndTimestamp = endTimestamp
		item.Month = startMonth
		ret = append(ret, &item)
		return ret
	}

	item := MonthTab{}
	item.TabName = "t_gd_access"

	i := endMonth
	loc, _ := time.LoadLocation("Local")
	for {
		tab := MonthTab{
			Month:   i,
			TabName: fmt.Sprintf("%s%d", prefix, i),
		}
		tab.StartTimestamp = time.Date(int(year), time.Month(i), 1, 0, 0, 0, 0, loc).Unix()
		if i+1 > 12 {
			tab.EndTimestamp = time.Date(int(year+1), time.Month(1), 1, 0, 0, 0, 0, loc).Unix()
		} else {
			tab.EndTimestamp = time.Date(int(year), time.Month(i+1), 1, 0, 0, 0, 0, loc).Unix()
		}
		if i == endMonth {
			tab.EndTimestamp = endTimestamp
		}
		if i == startMonth {
			tab.StartTimestamp = startTimestamp
		}
		ret = append(ret, &tab)
		if i == startMonth {
			break
		}
		i--
		if i <= 0 {
			i = 12
			year -= 1
		}
	}

	return ret
}

func LogExport(ctx context.Context, req *apis.LogQueryAccessLogReq, reply *apis.LogQueryAccessLogReply) error {
	if req.StartTimestamp <= 0 || req.EndTimestamp <= 0 {
		now := time.Now()
		req.StartTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
		req.EndTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 23, 59, 59, 0, now.Location()).Unix()
	}

	mTabs := getMonthTabs("t_gd_access_log_month", req.StartTimestamp, req.EndTimestamp)
	o := orm.NewOrm()
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	for _, v := range mTabs {
		mreq := *req
		mreq.StartTimestamp = v.StartTimestamp
		mreq.EndTimestamp = v.EndTimestamp
		sql := getAccesslogWhereSql(&mreq, v.TabName)
		querySql := fmt.Sprintf("%s and id>%d limit ?", sql, req.LastId)
		_, err := o.Raw(querySql, req.PageSize).QueryRows(&reply.AccessLogs)
		if err != nil && err != orm.ErrNoRows {
			return errors.DataBaseError
		}
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize

	return nil
}

func LogQueryAccessLogNew(req *apis.LogQueryAccessLogReq, reply *apis.LogQueryAccessLogReply) error {

	if req.StartTimestamp <= 0 || req.EndTimestamp <= 0 {
		now := time.Now()
		req.StartTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
		req.EndTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 23, 59, 59, 0, now.Location()).Unix()
	}

	mTabs := getMonthTabs("t_gd_access_log_month", req.StartTimestamp, req.EndTimestamp)
	o := orm.NewOrm()
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	oldCount := (req.PageNumber - 1) * req.PageSize
	for i, v := range mTabs {
		mreq := *req
		mreq.StartTimestamp = v.StartTimestamp
		mreq.EndTimestamp = v.EndTimestamp
		querySet := o.QueryTable(v.TabName)
		sql := getAccesslogWhereSql(&mreq, v.TabName)
		querySet = getAccesslogWhere(&mreq, querySet)
		count, err := querySet.Count()
		if err != nil {
			return errors.DataBaseError
		}
		mTabs[i].Count = count
		reply.Total += count
		if reply.Total <= oldCount {
			continue
		}
		list := []apis.AccessLog{}
		if int64(len(reply.AccessLogs)) < req.PageSize {
			offset := oldCount - (reply.Total - count)
			querySql := sql + " order by timestamp desc limit ? offset ?"
			if req.UseId {
				idSql := strings.Replace(sql, "*", "id", -1)
				querySql = fmt.Sprintf("%s and id>(%s limit ?, 1) limit ?", sql, idSql)
				_, err = o.Raw(querySql, offset, req.PageSize-int64(len(reply.AccessLogs))).QueryRows(&list)
			} else {
				_, err = o.Raw(querySql, req.PageSize-int64(len(reply.AccessLogs)), offset).QueryRows(&list)
			}

			if err != nil && err != orm.ErrNoRows {
				return errors.DataBaseError
			}
			reply.AccessLogs = append(reply.AccessLogs, list...)
			oldCount += int64(len(list))
		}
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	if req.IsExport {
		return nil
	}

	for i, v := range reply.AccessLogs {
		var timeLayout = "2006-01-02 15:04:05"
		reply.AccessLogs[i].CreateTime = time.Unix(reply.AccessLogs[i].Timestamp, 0).Format(timeLayout)
		if strings.Contains(v.ResponseParams, "{") {
			bytes := []byte(v.ResponseParams)
			m := map[string]interface{}{}
			json.Unmarshal(bytes, &m)
			bytes, err := json.Marshal(m)
			if err == nil {
				reply.AccessLogs[i].ResponseParams = string(bytes)
			}
			m = nil
		}
		if strings.Contains(v.RawResponseParams, "{") {
			bytes := []byte(v.RawResponseParams)
			m := map[string]interface{}{}
			json.Unmarshal(bytes, &m)
			bytes, err := json.Marshal(m)
			if err == nil {
				reply.AccessLogs[i].RawResponseParams = string(bytes)
			}
			m = nil
		}
		if strings.Contains(v.RequestParams, "{") {
			bytes := []byte(v.RequestParams)
			m := map[string]interface{}{}
			json.Unmarshal(bytes, &m)
			bytes, err := json.Marshal(m)
			if err == nil {
				reply.AccessLogs[i].RequestParams = string(bytes)
			}
			m = nil
		}
		if strings.Contains(v.RawRequestParams, "{") {
			bytes := []byte(v.RawRequestParams)
			m := map[string]interface{}{}
			json.Unmarshal(bytes, &m)
			bytes, err := json.Marshal(m)
			if err == nil {
				reply.AccessLogs[i].RawRequestParams = string(bytes)
			}
			m = nil
		}
	}

	return nil
}

func LogQueryAccessLog(ctx context.Context, req *apis.LogQueryAccessLogReq, reply *apis.LogQueryAccessLogReply) error {
	return LogQueryAccessLogNew(req, reply)
}
